<?php

namespace Drupal\widget_provider_example\Widget;

use Drupal\widget_provider_api\Widget\WidgetBase;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

/**
 * A Node Widget Example class.
 * It renders a Node as a widget.
 */
class WidgetDrupalNodeExample extends WidgetBase {
  /**
   * The WidgetDrupalNodeExample Constructor.
   */
  public function __construct() {
    parent::__construct('widget-node-example', ['nid'], ['kid']);
  }

  /**
   * Alternative build() with the Page Body still active.
   *
   * @return array
   */
  public function buildContent(): array {
    return [];
  }

  /**
   * The build function to create the Widget.
   *
   * @return void
   */
  public function build(){
    $nid = $this->getParameterValue('nid', false);
    $referer = $this->validateAllowedReferer();
    $validate = $this->validateParameters();
    if($nid === false || $referer === false || $validate === false){
      // Example for NOT FOUND exception return value:
      return false;
    }
    return $this->buildNode($nid);
  }

  /**
   * Node render function, for creating a node widget.
   *
   * @param string $entity_id
   * @return array
   * @throws AccessDeniedHttpException
   */
  public function buildNode($entity_id){
    $current_user = \Drupal::currentUser();
    $user = \Drupal\user\Entity\User::load($current_user->id());
    $entity = \Drupal::entityTypeManager()->getStorage('node')->load($entity_id);
    if($entity->access($user)){
      $view_builder = \Drupal::entityTypeManager()->getViewBuilder('node');
      $pre_render = $view_builder->view($entity, 'full');
      return $pre_render;
    }
    throw new AccessDeniedHttpException("You don't have access to this Node!!");
  }
}
