<?php

namespace Drupal\widget_provider_example\Widget;

use Drupal\widget_provider_api\Widget\WidgetBase;

/**
 * A Iframe Widget Example class.
 * It renders an Iframe as a widget.
 */
class WidgetIframeChecksumExample extends WidgetBase {
  /**
   * The WidgetIFrameExample Constructor.
   */
  public function __construct() {
    parent::__construct('widget-checksum-iframe-example', [], [], [], 'testSecret');
  }

  /**
   * The build function to create the Widget.
   *
   * @return void
   */
  public function build() {
    return $this->buildIframe('https://www.google.com/');
  }

  /**
   * Alternative build() with the Page Body still active.
   *
   * @return array
   */
  public function buildContent(): array {
    return [];
  }

  public function generateChecksum() {
    $secretID[0] = $this->getValidateChecksumSecret();
    $request = \Drupal::request();
    $time[0] = $request->server->get('REQUEST_TIME');
    $allParameters = $this->getAllParameters();
    $allowedReferrers = $this->getAllowedReferrers();
    $widgetData = array_merge($secretID, $time, $allParameters, $allowedReferrers);
    $widgetDataString = implode('', $widgetData);
    $hash = hash('sha256', $widgetDataString);
    return $hash;
  }

  /**
   * Function for rendering a iframe
   *
   * @param string $url
   * @return array
   */
  public function buildIframe($url) {
    return [
      '#type' => 'inline_template',
      '#template' => '<iframe src="{{ url }}"></iframe>',
      '#context' => [
        'url' => $url,
      ],
    ];
  }
}
