<?php

namespace Drupal\widget_provider_api\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Response;
use Drupal\widget_provider_api\Widget\WidgetApiManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\widget_provider_api\Exceptions\WidgetNotFoundException;
use Drupal\widget_provider_api\Exceptions\WidgetValidationException;
use Drupal\widget_provider_api\Exceptions\WidgetCreationFailedException;
use Drupal\widget_provider_api\Exceptions\WidgetProviderNotFoundException;
use Drupal\widget_provider_api\Exceptions\NoRegisteredWidgetParameterException;

/**
 * Returns responses for WidgetProviderAPI routes.
 */
class WidgetApiController extends ControllerBase {

  /**
   * A WidgetApiManager Object.
   *
   * @var Drupal\widget_provider_api\WidgetApiManager
   */
  protected $widgetApiManager;

  /**
   * Constructs a WidgetApiController object.
   *
   * @param Drupal\widget_provider_api\WidgetApiManager
   *   The module handler service.
   */
  public function __construct(WidgetApiManager $widgetApiManager) {
    $this->widgetApiManager = $widgetApiManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): object {
    return new static(
      $container->get('widget_provider_api.widget_manager')
    );
  }

  /**
   * Build the response.
   *
   * @param string $providerId
   * @param string $widgetId
   * @return array|Response
   */
  public function build($providerId, $widgetId) {
    try {
      return $this->widgetApiManager->build($providerId, $widgetId);
    } catch (WidgetCreationFailedException $e) {
      throw new NotFoundHttpException();
    } catch (WidgetValidationException $e) {
      throw new NotFoundHttpException();
    } catch (WidgetNotFoundException $e) {
      throw new NotFoundHttpException();
    } catch (WidgetProviderNotFoundException $e) {
      throw new NotFoundHttpException();
    } catch (NoRegisteredWidgetParameterException $e){
      throw new NotFoundHttpException();
    }
  }
}
