<?php

namespace Drupal\widget_provider_api\Widget;

interface WidgetInterface {
  /**
   * Retrieve the Widget id.
   *
   * @return string
   */
  public function getId(): string;

  /**
   * Retrieve required Parameters.
   *
   * @return array
   */
  public function getRequiredParameters(): array;

  /**
   * Retrieve optional Parameters.
   *
   * @return array
   */
  public function getOptionalParameters(): array;

  /**
   * Retrieve all Parameters.
   *
   * @return array
   */
  public function getAllParameters(): array;

  /**
   * Retrieve all Allowed Widget Referrers.
   *
   * @return array
   */
  public function getAllowedReferrers(): array;

  /**
   * Build the Response content render array.
   *
   * @return array
   */
  public function buildContent(): array;

  /**
   * Build the Response.
   *
   * @return array|Response
   */
  public function build();

  /**
   * Returns the value of the given registered $parameterKey. $default if empty.
   *
   * @param string $parameterKey
   * @param mixed $default The default if the parameter does not exist.
   * @throws NoRegisteredWidgetParameterException
   * @return mixed
   */
  public function getParameterValue($parameterKey, $default = null);
}
