<?php

namespace Drupal\widget_provider_api\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\widget_provider_api\Widget\WidgetApiManager;
use Drupal\widget_provider_api\Widget\BlankWidgetProvider;
use ReflectionClass;

/**
 * This class provides methods specifically for testing the WidgetApiManager.
 *
 * @group widget_provider_api
 */
class WidgetApiManagerUnitTest extends UnitTestCase {

  // Helper:
  // fwrite(STDERR, print_r($attr, TRUE));

  /**
   * A WidgetApiManager object.
   *
   * @var \Drupal\widget_provider_api\Widget\WidgetApiManager
   */
  protected $widgetApiManager;

  /**
   * Get an accessible WidgetApiManager ReflectionClass method by method name.
   *
   * @param [String] $name
   * @return void
   */
  protected static function getMethod($name) {
    $class = new ReflectionClass('\Drupal\widget_provider_api\Widget\WidgetApiManager');
    $method = $class->getMethod($name);
    $method->setAccessible(true);
    return $method;
  }

  /**
   * {@inheritDoc}
   *
   * @return void
   */
  protected function setUp(): void {
    parent::setUp();
    $eventdispatcher = $this->createMock('Symfony\Component\EventDispatcher\EventDispatcherInterface');
    $this->widgetApiManager = new WidgetApiManager($eventdispatcher);
  }

  /**
   * Tests if addWidgetProvider will add a widgetProvider.
   *
   * @return void
   */
  public function testAddWidgetProvider() {
    $attributeProviders = $this->widgetApiManager->getWidgetProviders();
    if (empty($attributeProviders)) {
      $this->assertTrue(true);
    } else {
      $this->assertTrue(false);
    }
    $widgetProvider = $this->createMock('Drupal\widget_provider_api\Widget\WidgetProviderInterface');
    $addWidgetProvider = self::getMethod('addWidgetProvider');
    $addWidgetProvider->invokeArgs($this->widgetApiManager, [$widgetProvider]);
    $attributeProviders = $this->widgetApiManager->getWidgetProviders();
    if (empty($attributeProviders)) {
      $this->assertTrue(false);
    } else {
      $this->assertTrue(true);
    }
  }

  /**
   * Tests if an unallowed provider id throws an error.
   *
   * @return void
   */
  public function testAddWidgetProviderIdNotAllowed() {
    $this->expectException(\Exception::class);
    $widgetProvider = new BlankWidgetProvider('()==)§!', []);
    $addWidgetProvider = self::getMethod('addWidgetProvider');
    $addWidgetProvider->invokeArgs($this->widgetApiManager, [$widgetProvider]);
  }

  /**
   * Tests if adding multiple of the same providers throws an error.
   *
   * @return void
   */
  public function testAddWidgetProviderAddMultiple() {
    $this->expectException(\Exception::class);
    $widgetProvider = new BlankWidgetProvider('test1', []);
    $widgetProvider2 = new BlankWidgetProvider('test1', []);
    $addWidgetProvider = self::getMethod('addWidgetProvider');
    $addWidgetProvider->invokeArgs($this->widgetApiManager, [$widgetProvider]);
    $addWidgetProvider->invokeArgs($this->widgetApiManager, [$widgetProvider2]);
  }
}
