<?php

namespace Drupal\widget_provider_api\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\widget_provider_api\Widget\BlankWidgetProvider;
use Drupal\widget_provider_api\Widget\BlankWidget;
use ReflectionClass;

/**
 * This class provides methods specifically for testing the WidgetProviderBase.
 *
 * @group widget_provider_api
 */
class WidgetProviderBaseUnitTest extends UnitTestCase {
  /**
   * A BlankWidgetProvider Object.
   *
   * @var Drupal\widget_provider_api\Widget\BlankWidgetProvider
   */
  protected $widgetProvider;

  /**
   * {@inheritDoc}
   */
  protected function setUp(): void {
    $this->widgetProvider = new BlankWidgetProvider('testProvider', []);
  }

  /**
   * Tests if it is possible to add a Widget.
   */
  public function testAddWidget() {
    $widget = new BlankWidget('testWidget');
    $addWidget = self::getMethod('addWidget');
    $addWidget->invokeArgs($this->widgetProvider, [$widget]);
    $widgetArray = $this->widgetProvider->getWidgets();
    $this->assertArrayHasKey('testWidget', $widgetArray);
  }

  /**
   * Tests if an unallowed widget id throws an error.
   */
  public function testAddWidgetIdNotAllowed() {
    $this->expectException(\Exception::class);
    $widget = new BlankWidget('&%(/%');
    $addWidget = self::getMethod('addWidget');
    $addWidget->invokeArgs($this->widgetProvider, [$widget]);
  }

  /**
   * Tests if adding multiple of the same widgets throws an error.
   */
  public function testAddWidgetProviderAddMultiple() {
    $this->expectException(\Exception::class);
    $widget = new BlankWidget('testWidget');
    $widget2 = new BlankWidget('testWidget');
    $addWidget = self::getMethod('addWidget');
    $addWidget->invokeArgs($this->widgetProvider, [$widget]);
    $addWidget->invokeArgs($this->widgetProvider, [$widget2]);
  }

  /**
   * Get an accessible BlankWidgetProvider ReflectionClass method by method name.
   *
   * @param string $name
   */
  protected static function getMethod($name) {
    $class = new ReflectionClass('\Drupal\widget_provider_api\Widget\BlankWidgetProvider');
    $method = $class->getMethod($name);
    $method->setAccessible(true);

    return $method;
  }
}
